
/**
 * Copyright © 2020, Oracle and/or its affiliates. All rights reserved.
 * 
 * Server side JWT token sample for use with Oracle Web SDK in
 * Oracle Digital Assistant 19.10 and later. 
 * Credentials and keys issued with the channel are highly
 * sensitive. You should use channel credentials 
 * and keys to generate the JWT tokens on the server side and
 * never on the client -- web or mobile.
 * 
 * The Universal Permissive License (UPL), Version 1.0
 * @author Based on sample provided by Frank Nimphius
 */

var express = require('express');
var router = express.Router();

const fs = require('fs');
const path = require('path');
const jwt = require('jsonwebtoken');

// request <server>/token?config=<conf-name>&userId=<user-id>. 
// The server should use https for security.
router.get('/', function (req, res, next) {

  const configFile = fs.readFileSync(__dirname + path.sep + 'config.json');
  var configParam = req.query.config;

  // require config value
  if (!configParam) {
    console.log('config query parameter missing in request');
    res.status(404).send('Not found');
  }

  if (!configFile) {
    console.log('./config.json not found');
    res.status(500).send('Missing server side configuration');
  }

  try {
    let configSettings = JSON.parse(configFile);

    var confObject = configSettings.find(function (item) {
      return item.name == configParam;
    });
    console.log('configuration found: ' + (confObject != null ? 'true' : 'false'));
    if (confObject) {
      var payload = {};
      var userIdParam = (req.query.userId) ?
        req.query.userId : null;
      // This code to generate a user ID is for 
      // use in the context of the tutorial and
      // is not intended for production

      // If userId query parameter passed in and
      // ignoreExternalUserId is set to false,
      // USE USERID FROM REQUEST

      // If no userId query parameter or if ignoreExternalUserId 
      // is set to true or is missing, then use 
      // GENERATE RANDOM USERID
      if (userIdParam && confObject.config.ignoreExternalUserId == false) {

        console.log('userId query parameter passed '
        + 'and ignoreExternalUserId is FALSE. '
        + 'Using userId query parameter.');            
        payload.userId = userIdParam;            
      } else {
        if (userIdParam) {
          console.log('userId query parameter passed, '
            + 'but config.json ignoreExternalUserId is TRUE or is missing. '
            + 'Ignoring userId query parameter and generating user ID.');          
        } else {
          console.log('Either no userId query parameter, '
            + 'ignoreExternalUserId is set to true, or it is missing. '
            + 'Generating user ID.');
        }
        payload.userId = getRandomArbitrary(1000, 9999999);      
      }
      console.log('userId set to: ' + payload.userId);

      payload.channelId = confObject.config.channelId;

      // Create and return JWT token. If expiry time is not set
      // in configuration file, set it to 1 hour.    
      try {
        var token = jwt.sign(payload, confObject.config.channelSecret, { algorithm: 'HS256', expiresIn: confObject.config.exp });
        res.status(200).send({ 'token': token });
      }
      catch (e) {
        console.log('Could not sign payload \n\n' + e);
        res.status(500).send('Internal error');
      }
    }
  } catch (e) {
    console.log('Failed parsing ./config.json. \n\n' + e);
    res.status(500).send('Internal error');
  }
});

function getRandomArbitrary(min, max) {
  min = Math.ceil(min);
  max = Math.floor(max);
  return (Math.floor(Math.random() * (max - min + 1)) + min) + 'usr' + (new Date()).getTime();
}

module.exports = router;
